@@ -139,7 +139,8 @@ arrow_with_json <- function() {
 some_features_are_off <- function(features) {
   # `features` is a named logical vector (as in arrow_info()$capabilities)
   # Let's exclude some less relevant ones
-  blocklist <- c("lzo", "bz2", "brotli", "substrait")
+  # jemalloc is only included because it is sometimes disabled in our build process
+  blocklist <- c("lzo", "bz2", "brotli", "substrait", "jemalloc")
   # Return TRUE if any of the other features are FALSE
   !all(features[setdiff(names(features), blocklist)])
 }
