@@ -27,7 +27,7 @@
 #include "arrow/acero/options.h"
 #include "arrow/acero/schema_util.h"
 #include "arrow/acero/util.h"
-#include "arrow/compute/key_hash.h"
+#include "arrow/compute/key_hash_internal.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/future.h"
 #include "arrow/util/thread_pool.h"
@@ -236,14 +236,14 @@ Status HashJoinSchema::ValidateSchemas(JoinType join_type, const Schema& left_sc
     const auto& type = *field->type();
     if (!IsTypeSupported(type)) {
       return Status::Invalid("Data type ", type,
-                             " is not supported in join non-key field");
+                             " is not supported in join non-key field ", field->name());
     }
   }
   for (const auto& field : right_schema.fields()) {
     const auto& type = *field->type();
     if (!IsTypeSupported(type)) {
       return Status::Invalid("Data type ", type,
-                             " is not supported in join non-key field");
+                             " is not supported in join non-key field ", field->name());
     }
   }
 
@@ -740,13 +740,11 @@ class HashJoinNode : public ExecNode, public TracedNode {
     // Create hash join implementation object
     // SwissJoin does not support:
     // a) 64-bit string offsets
-    // b) residual predicates
-    // c) dictionaries
+    // b) dictionaries
     //
     bool use_swiss_join;
 #if ARROW_LITTLE_ENDIAN
-    use_swiss_join = (filter == literal(true)) && !schema_mgr->HasDictionaries() &&
-                     !schema_mgr->HasLargeBinary();
+    use_swiss_join = !schema_mgr->HasDictionaries() && !schema_mgr->HasLargeBinary();
 #else
     use_swiss_join = false;
 #endif
