@@ -18,17 +18,17 @@
 #include "arrow/adapters/orc/adapter.h"
 
 #include <algorithm>
-#include <cstdint>
-#include <functional>
 #include <list>
 #include <memory>
 #include <sstream>
 #include <string>
-#include <utility>
 #include <vector>
 
+#ifdef ARROW_ORC_NEED_TIME_ZONE_DATABASE_CHECK
+#include <filesystem>
+#endif
+
 #include "arrow/adapters/orc/util.h"
-#include "arrow/buffer.h"
 #include "arrow/builder.h"
 #include "arrow/io/interfaces.h"
 #include "arrow/memory_pool.h"
@@ -37,14 +37,11 @@
 #include "arrow/table.h"
 #include "arrow/table_builder.h"
 #include "arrow/type.h"
-#include "arrow/type_traits.h"
 #include "arrow/util/bit_util.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/decimal.h"
 #include "arrow/util/key_value_metadata.h"
 #include "arrow/util/macros.h"
-#include "arrow/util/range.h"
-#include "arrow/util/visibility.h"
 #include "orc/Exceptions.hh"
 
 // alias to not interfere with nested orc namespace
@@ -80,6 +77,12 @@ namespace liborc = orc;
   }                                            \
   catch (const liborc::NotImplementedYet& e) { \
     return Status::NotImplemented(e.what());   \
+  }                                            \
+  catch (const std::exception& e) {            \
+    return Status::UnknownError(e.what());     \
+  }                                            \
+  catch (...) {                                \
+    return Status::UnknownError("ORC error");  \
   }
 
 #define ORC_CATCH_NOT_OK(_s)  \
@@ -173,7 +176,7 @@ class OrcStripeReader : public RecordBatchReader {
   int64_t batch_size_;
 };
 
-liborc::RowReaderOptions default_row_reader_options() {
+liborc::RowReaderOptions DefaultRowReaderOptions() {
   liborc::RowReaderOptions options;
   // Orc timestamp type is error-prone since it serializes values in the writer timezone
   // and reads them back in the reader timezone. To avoid this, both the Apache Orc C++
@@ -183,6 +186,23 @@ liborc::RowReaderOptions default_row_reader_options() {
   return options;
 }
 
+#ifdef ARROW_ORC_NEED_TIME_ZONE_DATABASE_CHECK
+// Proactively check timezone database availability for ORC versions older than 2.0.0
+Status CheckTimeZoneDatabaseAvailability() {
+  auto tz_dir = std::getenv("TZDIR");
+  bool is_tzdb_avaiable = tz_dir != nullptr
+                              ? std::filesystem::exists(tz_dir)
+                              : std::filesystem::exists("/usr/share/zoneinfo");
+  if (!is_tzdb_avaiable) {
+    return Status::Invalid(
+        "IANA time zone database is unavailable but required by ORC."
+        " Please install it to /usr/share/zoneinfo or set TZDIR env to the installed"
+        " directory");
+  }
+  return Status::OK();
+}
+#endif
+
 }  // namespace
 
 class ORCFileReader::Impl {
@@ -332,25 +352,25 @@ class ORCFileReader::Impl {
   }
 
   Result<std::shared_ptr<Table>> Read() {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema());
     return ReadTable(opts, schema);
   }
 
   Result<std::shared_ptr<Table>> Read(const std::shared_ptr<Schema>& schema) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     return ReadTable(opts, schema);
   }
 
   Result<std::shared_ptr<Table>> Read(const std::vector<int>& include_indices) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectIndices(&opts, include_indices));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
     return ReadTable(opts, schema);
   }
 
   Result<std::shared_ptr<Table>> Read(const std::vector<std::string>& include_names) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectNames(&opts, include_names));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
     return ReadTable(opts, schema);
@@ -358,13 +378,13 @@ class ORCFileReader::Impl {
 
   Result<std::shared_ptr<Table>> Read(const std::shared_ptr<Schema>& schema,
                                       const std::vector<int>& include_indices) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectIndices(&opts, include_indices));
     return ReadTable(opts, schema);
   }
 
   Result<std::shared_ptr<RecordBatch>> ReadStripe(int64_t stripe) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectStripe(&opts, stripe));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
     return ReadBatch(opts, schema, stripes_[static_cast<size_t>(stripe)].num_rows);
@@ -372,7 +392,7 @@ class ORCFileReader::Impl {
 
   Result<std::shared_ptr<RecordBatch>> ReadStripe(
       int64_t stripe, const std::vector<int>& include_indices) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectIndices(&opts, include_indices));
     RETURN_NOT_OK(SelectStripe(&opts, stripe));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
@@ -381,7 +401,7 @@ class ORCFileReader::Impl {
 
   Result<std::shared_ptr<RecordBatch>> ReadStripe(
       int64_t stripe, const std::vector<std::string>& include_names) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     RETURN_NOT_OK(SelectNames(&opts, include_names));
     RETURN_NOT_OK(SelectStripe(&opts, stripe));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
@@ -487,11 +507,11 @@ class ORCFileReader::Impl {
       return nullptr;
     }
 
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     if (!include_indices.empty()) {
       RETURN_NOT_OK(SelectIndices(&opts, include_indices));
     }
-    StripeInformation stripe_info({0, 0, 0, 0});
+    StripeInformation stripe_info{0, 0, 0, 0};
     RETURN_NOT_OK(SelectStripeWithRowNumber(&opts, current_row_, &stripe_info));
     ARROW_ASSIGN_OR_RAISE(auto schema, ReadSchema(opts));
     std::unique_ptr<liborc::RowReader> row_reader;
@@ -508,7 +528,7 @@ class ORCFileReader::Impl {
 
   Result<std::shared_ptr<RecordBatchReader>> GetRecordBatchReader(
       int64_t batch_size, const std::vector<std::string>& include_names) {
-    liborc::RowReaderOptions opts = default_row_reader_options();
+    liborc::RowReaderOptions opts = DefaultRowReaderOptions();
     if (!include_names.empty()) {
       RETURN_NOT_OK(SelectNames(&opts, include_names));
     }
@@ -541,6 +561,9 @@ ORCFileReader::~ORCFileReader() {}
 
 Result<std::unique_ptr<ORCFileReader>> ORCFileReader::Open(
     const std::shared_ptr<io::RandomAccessFile>& file, MemoryPool* pool) {
+#ifdef ARROW_ORC_NEED_TIME_ZONE_DATABASE_CHECK
+  RETURN_NOT_OK(CheckTimeZoneDatabaseAvailability());
+#endif
   auto result = std::unique_ptr<ORCFileReader>(new ORCFileReader());
   RETURN_NOT_OK(result->impl_->Open(file, pool));
   return std::move(result);
@@ -779,7 +802,7 @@ class ORCFileWriter::Impl {
             &(arrow_index_offset[i]), (root->fields)[i]));
       }
       root->numElements = (root->fields)[0]->numElements;
-      writer_->add(*batch);
+      ORC_CATCH_NOT_OK(writer_->add(*batch));
       batch->clear();
       num_rows -= batch_size;
     }
@@ -807,6 +830,9 @@ ORCFileWriter::ORCFileWriter() { impl_.reset(new ORCFileWriter::Impl()); }
 
 Result<std::unique_ptr<ORCFileWriter>> ORCFileWriter::Open(
     io::OutputStream* output_stream, const WriteOptions& writer_options) {
+#ifdef ARROW_ORC_NEED_TIME_ZONE_DATABASE_CHECK
+  RETURN_NOT_OK(CheckTimeZoneDatabaseAvailability());
+#endif
   std::unique_ptr<ORCFileWriter> result =
       std::unique_ptr<ORCFileWriter>(new ORCFileWriter());
   Status status = result->impl_->Open(output_stream, writer_options);
