@@ -87,13 +87,15 @@ struct ScalarFromArraySlotImpl {
     return Finish(a.GetString(index_));
   }
 
+  Status Visit(const BinaryViewArray& a) { return Finish(a.GetString(index_)); }
+
   Status Visit(const FixedSizeBinaryArray& a) { return Finish(a.GetString(index_)); }
 
   Status Visit(const DayTimeIntervalArray& a) { return Finish(a.Value(index_)); }
   Status Visit(const MonthDayNanoIntervalArray& a) { return Finish(a.Value(index_)); }
 
   template <typename T>
-  Status Visit(const BaseListArray<T>& a) {
+  Status Visit(const VarLengthListLikeArray<T>& a) {
     return Finish(a.value_slice(index_));
   }
 
@@ -305,6 +307,18 @@ Result<std::shared_ptr<Array>> Array::View(
   return MakeArray(result);
 }
 
+Result<std::shared_ptr<Array>> Array::CopyTo(
+    const std::shared_ptr<MemoryManager>& to) const {
+  ARROW_ASSIGN_OR_RAISE(auto copied_data, data()->CopyTo(to));
+  return MakeArray(copied_data);
+}
+
+Result<std::shared_ptr<Array>> Array::ViewOrCopyTo(
+    const std::shared_ptr<MemoryManager>& to) const {
+  ARROW_ASSIGN_OR_RAISE(auto new_data, data()->ViewOrCopyTo(to));
+  return MakeArray(new_data);
+}
+
 // ----------------------------------------------------------------------
 // NullArray
 
