@@ -29,11 +29,14 @@
 #include "arrow/builder.h"
 #include "arrow/memory_pool.h"
 #include "arrow/testing/gtest_util.h"
+#include "arrow/util/benchmark_util.h"
 #include "arrow/util/bit_util.h"
 #include "arrow/util/decimal.h"
 
 namespace arrow {
 
+::arrow::BenchmarkMemoryTracker memory_tracker;
+
 using ValueType = int64_t;
 using VectorType = std::vector<ValueType>;
 
@@ -53,13 +56,14 @@ constexpr int64_t kRounds = 256;
 static VectorType kData = AlmostU8CompressibleVector();
 constexpr int64_t kBytesProcessPerRound = kNumberOfElements * sizeof(ValueType);
 constexpr int64_t kBytesProcessed = kRounds * kBytesProcessPerRound;
+constexpr int64_t kItemsProcessed = kRounds * kNumberOfElements;
 
 static const char* kBinaryString = "12345678";
 static std::string_view kBinaryView(kBinaryString);
 
 static void BuildIntArrayNoNulls(benchmark::State& state) {  // NOLINT non-const reference
   for (auto _ : state) {
-    Int64Builder builder;
+    Int64Builder builder(memory_tracker.memory_pool());
 
     for (int i = 0; i < kRounds; i++) {
       ABORT_NOT_OK(builder.AppendValues(kData.data(), kData.size(), nullptr));
@@ -70,12 +74,13 @@ static void BuildIntArrayNoNulls(benchmark::State& state) {  // NOLINT non-const
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildAdaptiveIntNoNulls(
     benchmark::State& state) {  // NOLINT non-const reference
   for (auto _ : state) {
-    AdaptiveIntBuilder builder;
+    AdaptiveIntBuilder builder(memory_tracker.memory_pool());
 
     for (int i = 0; i < kRounds; i++) {
       ABORT_NOT_OK(builder.AppendValues(kData.data(), kData.size(), nullptr));
@@ -86,12 +91,13 @@ static void BuildAdaptiveIntNoNulls(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildAdaptiveIntNoNullsScalarAppend(
     benchmark::State& state) {  // NOLINT non-const reference
   for (auto _ : state) {
-    AdaptiveIntBuilder builder;
+    AdaptiveIntBuilder builder(memory_tracker.memory_pool());
 
     for (int i = 0; i < kRounds; i++) {
       for (size_t j = 0; j < kData.size(); j++) {
@@ -104,6 +110,7 @@ static void BuildAdaptiveIntNoNullsScalarAppend(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildBooleanArrayNoNulls(
@@ -113,7 +120,7 @@ static void BuildBooleanArrayNoNulls(
   const uint8_t* data = reinterpret_cast<const uint8_t*>(kData.data());
 
   for (auto _ : state) {
-    BooleanBuilder builder;
+    BooleanBuilder builder(memory_tracker.memory_pool());
 
     for (int i = 0; i < kRounds; i++) {
       ABORT_NOT_OK(builder.AppendValues(data, n_bytes));
@@ -124,11 +131,12 @@ static void BuildBooleanArrayNoNulls(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildBinaryArray(benchmark::State& state) {  // NOLINT non-const reference
   for (auto _ : state) {
-    BinaryBuilder builder;
+    BinaryBuilder builder(memory_tracker.memory_pool());
 
     for (int64_t i = 0; i < kRounds * kNumberOfElements; i++) {
       ABORT_NOT_OK(builder.Append(kBinaryView));
@@ -139,6 +147,7 @@ static void BuildBinaryArray(benchmark::State& state) {  // NOLINT non-const ref
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildChunkedBinaryArray(
@@ -147,7 +156,7 @@ static void BuildChunkedBinaryArray(
   const int32_t kChunkSize = 1 << 20;
 
   for (auto _ : state) {
-    internal::ChunkedBinaryBuilder builder(kChunkSize);
+    internal::ChunkedBinaryBuilder builder(kChunkSize, memory_tracker.memory_pool());
 
     for (int64_t i = 0; i < kRounds * kNumberOfElements; i++) {
       ABORT_NOT_OK(builder.Append(kBinaryView));
@@ -158,6 +167,7 @@ static void BuildChunkedBinaryArray(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildFixedSizeBinaryArray(
@@ -165,7 +175,7 @@ static void BuildFixedSizeBinaryArray(
   auto type = fixed_size_binary(static_cast<int32_t>(kBinaryView.size()));
 
   for (auto _ : state) {
-    FixedSizeBinaryBuilder builder(type);
+    FixedSizeBinaryBuilder builder(type, memory_tracker.memory_pool());
 
     for (int64_t i = 0; i < kRounds * kNumberOfElements; i++) {
       ABORT_NOT_OK(builder.Append(kBinaryView));
@@ -176,6 +186,7 @@ static void BuildFixedSizeBinaryArray(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 static void BuildDecimalArray(benchmark::State& state) {  // NOLINT non-const reference
@@ -185,7 +196,7 @@ static void BuildDecimalArray(benchmark::State& state) {  // NOLINT non-const re
   int32_t scale = 0;
   ABORT_NOT_OK(Decimal128::FromString("1234.1234", &value, &precision, &scale));
   for (auto _ : state) {
-    Decimal128Builder builder(type);
+    Decimal128Builder builder(type, memory_tracker.memory_pool());
 
     for (int64_t i = 0; i < kRounds * kNumberOfElements; i++) {
       ABORT_NOT_OK(builder.Append(value));
@@ -196,6 +207,7 @@ static void BuildDecimalArray(benchmark::State& state) {  // NOLINT non-const re
   }
 
   state.SetBytesProcessed(state.iterations() * kRounds * kNumberOfElements * 16);
+  state.SetItemsProcessed(state.iterations() * kRounds * kNumberOfElements);
 }
 
 // ----------------------------------------------------------------------
@@ -298,7 +310,7 @@ static void BenchmarkDictionaryArray(
     benchmark::State& state,  // NOLINT non-const reference
     const std::vector<Scalar>& fodder, size_t fodder_nbytes = 0) {
   for (auto _ : state) {
-    DictionaryBuilderType builder(default_memory_pool());
+    DictionaryBuilderType builder(memory_tracker.memory_pool());
 
     for (int64_t i = 0; i < kRounds; i++) {
       for (const auto& value : fodder) {
@@ -314,6 +326,7 @@ static void BenchmarkDictionaryArray(
     fodder_nbytes = fodder.size() * sizeof(Scalar);
   }
   state.SetBytesProcessed(state.iterations() * fodder_nbytes * kRounds);
+  state.SetItemsProcessed(state.iterations() * fodder.size() * kRounds);
 }
 
 static void BuildInt64DictionaryArrayRandom(
@@ -358,6 +371,7 @@ static void ArrayDataConstructDestruct(
     InitArrays();
     arrays.clear();
   }
+  state.SetItemsProcessed(state.iterations() * kNumArrays);
 }
 
 // ----------------------------------------------------------------------
@@ -371,7 +385,7 @@ static void BenchmarkBufferBuilder(
   // Write approx. 256 MB to BufferBuilder
   int64_t num_raw_values = (1 << 28) / raw_nbytes;
   for (auto _ : state) {
-    BufferBuilder builder;
+    BufferBuilder builder(memory_tracker.memory_pool());
     std::shared_ptr<Buffer> buf;
     for (int64_t i = 0; i < num_raw_values; ++i) {
       ABORT_NOT_OK(builder.Append(raw_data, raw_nbytes));
@@ -427,6 +441,7 @@ static void ReferenceBuildVectorNoNulls(
   }
 
   state.SetBytesProcessed(state.iterations() * kBytesProcessed);
+  state.SetItemsProcessed(state.iterations() * kItemsProcessed);
 }
 
 BENCHMARK(ReferenceBuildVectorNoNulls);
