@@ -536,60 +536,67 @@ Result<Expression> BindNonRecursive(Expression::Call call, bool insert_implicit_
   std::vector<TypeHolder> types = GetTypes(call.arguments);
   ARROW_ASSIGN_OR_RAISE(call.function, GetFunction(call, exec_context));
 
+  auto FinishBind = [&] {
+    compute::KernelContext kernel_context(exec_context, call.kernel);
+    if (call.kernel->init) {
+      const FunctionOptions* options =
+          call.options ? call.options.get() : call.function->default_options();
+      ARROW_ASSIGN_OR_RAISE(
+          call.kernel_state,
+          call.kernel->init(&kernel_context, {call.kernel, types, options}));
+
+      kernel_context.SetState(call.kernel_state.get());
+    }
+
+    ARROW_ASSIGN_OR_RAISE(
+        call.type, call.kernel->signature->out_type().Resolve(&kernel_context, types));
+    return Status::OK();
+  };
+
   // First try and bind exactly
   Result<const Kernel*> maybe_exact_match = call.function->DispatchExact(types);
   if (maybe_exact_match.ok()) {
     call.kernel = *maybe_exact_match;
-  } else {
-    if (!insert_implicit_casts) {
-      return maybe_exact_match.status();
-    }
-    // If exact binding fails, and we are allowed to cast, then prefer casting literals
-    // first.  Since DispatchBest generally prefers up-casting the best way to do this is
-    // first down-cast the literals as much as possible
-    types = GetTypesWithSmallestLiteralRepresentation(call.arguments);
-    ARROW_ASSIGN_OR_RAISE(call.kernel, call.function->DispatchBest(&types));
-
-    for (size_t i = 0; i < types.size(); ++i) {
-      if (types[i] == call.arguments[i].type()) continue;
-
-      if (const Datum* lit = call.arguments[i].literal()) {
-        ARROW_ASSIGN_OR_RAISE(Datum new_lit,
-                              compute::Cast(*lit, types[i].GetSharedPtr()));
-        call.arguments[i] = literal(std::move(new_lit));
-        continue;
-      }
-
-      // construct an implicit cast Expression with which to replace this argument
-      Expression::Call implicit_cast;
-      implicit_cast.function_name = "cast";
-      implicit_cast.arguments = {std::move(call.arguments[i])};
-
-      // TODO(wesm): Use TypeHolder in options
-      implicit_cast.options = std::make_shared<compute::CastOptions>(
-          compute::CastOptions::Safe(types[i].GetSharedPtr()));
-
-      ARROW_ASSIGN_OR_RAISE(
-          call.arguments[i],
-          BindNonRecursive(std::move(implicit_cast),
-                           /*insert_implicit_casts=*/false, exec_context));
+    if (FinishBind().ok()) {
+      return Expression(std::move(call));
     }
   }
 
-  compute::KernelContext kernel_context(exec_context, call.kernel);
-  if (call.kernel->init) {
-    const FunctionOptions* options =
-        call.options ? call.options.get() : call.function->default_options();
+  if (!insert_implicit_casts) {
+    return maybe_exact_match.status();
+  }
+
+  // If exact binding fails, and we are allowed to cast, then prefer casting literals
+  // first.  Since DispatchBest generally prefers up-casting the best way to do this is
+  // first down-cast the literals as much as possible
+  types = GetTypesWithSmallestLiteralRepresentation(call.arguments);
+  ARROW_ASSIGN_OR_RAISE(call.kernel, call.function->DispatchBest(&types));
+
+  for (size_t i = 0; i < types.size(); ++i) {
+    if (types[i] == call.arguments[i].type()) continue;
+
+    if (const Datum* lit = call.arguments[i].literal()) {
+      ARROW_ASSIGN_OR_RAISE(Datum new_lit, compute::Cast(*lit, types[i].GetSharedPtr()));
+      call.arguments[i] = literal(std::move(new_lit));
+      continue;
+    }
+
+    // construct an implicit cast Expression with which to replace this argument
+    Expression::Call implicit_cast;
+    implicit_cast.function_name = "cast";
+    implicit_cast.arguments = {std::move(call.arguments[i])};
+
+    // TODO(wesm): Use TypeHolder in options
+    implicit_cast.options = std::make_shared<compute::CastOptions>(
+        compute::CastOptions::Safe(types[i].GetSharedPtr()));
+
     ARROW_ASSIGN_OR_RAISE(
-        call.kernel_state,
-        call.kernel->init(&kernel_context, {call.kernel, types, options}));
-
-    kernel_context.SetState(call.kernel_state.get());
+        call.arguments[i],
+        BindNonRecursive(std::move(implicit_cast),
+                         /*insert_implicit_casts=*/false, exec_context));
   }
 
-  ARROW_ASSIGN_OR_RAISE(
-      call.type, call.kernel->signature->out_type().Resolve(&kernel_context, types));
-
+  RETURN_NOT_OK(FinishBind());
   return Expression(std::move(call));
 }
 
@@ -838,6 +845,8 @@ Result<Expression> FoldConstants(Expression expr) {
       std::move(expr), [](Expression expr) { return expr; },
       [](Expression expr, ...) -> Result<Expression> {
         auto call = CallNotNull(expr);
+        if (!call->function->is_pure()) return expr;
+
         if (std::all_of(call->arguments.begin(), call->arguments.end(),
                         [](const Expression& argument) { return argument.literal(); })) {
           // all arguments are literal; we can evaluate this subexpression *now*
@@ -854,10 +863,6 @@ Result<Expression> FoldConstants(Expression expr) {
         if (GetNullHandling(*call) == compute::NullHandling::INTERSECTION) {
           // kernels which always produce intersected validity can be resolved
           // to null *now* if any of their inputs is a null literal
-          if (!call->type.type) {
-            return Status::Invalid("Cannot fold constants for unbound expression ",
-                                   expr.ToString());
-          }
           for (const Expression& argument : call->arguments) {
             if (argument.IsNullLiteral()) {
               if (argument.type()->Equals(*call->type.type)) {
@@ -1080,6 +1085,7 @@ Result<Expression> Canonicalize(Expression expr, compute::ExecContext* exec_cont
       [&AlreadyCanonicalized, exec_context](Expression expr) -> Result<Expression> {
         auto call = expr.call();
         if (!call) return expr;
+        if (!call->function->is_pure()) return expr;
 
         if (AlreadyCanonicalized(expr)) return expr;
 
