@@ -32,6 +32,7 @@
 #include "arrow/result.h"
 #include "arrow/status.h"
 #include "arrow/table.h"
+#include "arrow/util/future.h"
 #include "arrow/util/logging.h"
 
 #include "arrow/flight/client_auth.h"
@@ -39,11 +40,48 @@
 #include "arrow/flight/transport.h"
 #include "arrow/flight/transport/grpc/grpc_client.h"
 #include "arrow/flight/types.h"
+#include "arrow/flight/types_async.h"
 
 namespace arrow {
 
 namespace flight {
 
+namespace {
+template <typename T>
+class UnaryUnaryAsyncListener : public AsyncListener<T> {
+ public:
+  UnaryUnaryAsyncListener() : future_(arrow::Future<T>::Make()) {}
+
+  void OnNext(T result) override {
+    DCHECK(!result_.ok());
+    result_ = std::move(result);
+  }
+
+  void OnFinish(Status status) override {
+    if (status.ok()) {
+      DCHECK(result_.ok());
+    } else {
+      // Default-initialized result is not ok
+      DCHECK(!result_.ok());
+      result_ = std::move(status);
+    }
+    future_.MarkFinished(std::move(result_));
+  }
+
+  static std::pair<std::shared_ptr<AsyncListener<T>>, arrow::Future<T>> Make() {
+    auto self = std::make_shared<UnaryUnaryAsyncListener<T>>();
+    // Keep the listener alive by stashing it in the future
+    self->future_.AddCallback([self](const arrow::Result<T>&) {});
+    auto future = self->future_;
+    return std::make_pair(std::move(self), std::move(future));
+  }
+
+ private:
+  arrow::Result<T> result_;
+  arrow::Future<T> future_;
+};
+}  // namespace
+
 const char* kWriteSizeDetailTypeId = "flight::FlightWriteSizeStatusDetail";
 
 FlightCallOptions::FlightCallOptions()
@@ -79,16 +117,6 @@ arrow::Result<std::shared_ptr<Table>> FlightStreamReader::ToTable(
   return Table::FromRecordBatches(schema, std::move(batches));
 }
 
-Status FlightStreamReader::ReadAll(std::vector<std::shared_ptr<RecordBatch>>* batches,
-                                   const StopToken& stop_token) {
-  return ToRecordBatches(stop_token).Value(batches);
-}
-
-Status FlightStreamReader::ReadAll(std::shared_ptr<Table>* table,
-                                   const StopToken& stop_token) {
-  return ToTable(stop_token).Value(table);
-}
-
 /// \brief An ipc::MessageReader adapting the Flight ClientDataStream interface.
 ///
 /// In order to support app_metadata and reuse the existing IPC
@@ -520,11 +548,6 @@ arrow::Result<std::unique_ptr<FlightClient>> FlightClient::Connect(
   return Connect(location, FlightClientOptions::Defaults());
 }
 
-Status FlightClient::Connect(const Location& location,
-                             std::unique_ptr<FlightClient>* client) {
-  return Connect(location, FlightClientOptions::Defaults()).Value(client);
-}
-
 arrow::Result<std::unique_ptr<FlightClient>> FlightClient::Connect(
     const Location& location, const FlightClientOptions& options) {
   flight::transport::grpc::InitializeFlightGrpcClient();
@@ -538,11 +561,6 @@ arrow::Result<std::unique_ptr<FlightClient>> FlightClient::Connect(
   return client;
 }
 
-Status FlightClient::Connect(const Location& location, const FlightClientOptions& options,
-                             std::unique_ptr<FlightClient>* client) {
-  return Connect(location, options).Value(client);
-}
-
 Status FlightClient::Authenticate(const FlightCallOptions& options,
                                   std::unique_ptr<ClientAuthHandler> auth_handler) {
   RETURN_NOT_OK(CheckOpen());
@@ -564,11 +582,6 @@ arrow::Result<std::unique_ptr<ResultStream>> FlightClient::DoAction(
   return results;
 }
 
-Status FlightClient::DoAction(const FlightCallOptions& options, const Action& action,
-                              std::unique_ptr<ResultStream>* results) {
-  return DoAction(options, action).Value(results);
-}
-
 arrow::Result<CancelFlightInfoResult> FlightClient::CancelFlightInfo(
     const FlightCallOptions& options, const CancelFlightInfoRequest& request) {
   ARROW_ASSIGN_OR_RAISE(auto body, request.SerializeToString());
@@ -601,11 +614,6 @@ arrow::Result<std::vector<ActionType>> FlightClient::ListActions(
   return actions;
 }
 
-Status FlightClient::ListActions(const FlightCallOptions& options,
-                                 std::vector<ActionType>* actions) {
-  return ListActions(options).Value(actions);
-}
-
 arrow::Result<std::unique_ptr<FlightInfo>> FlightClient::GetFlightInfo(
     const FlightCallOptions& options, const FlightDescriptor& descriptor) {
   std::unique_ptr<FlightInfo> info;
@@ -614,10 +622,30 @@ arrow::Result<std::unique_ptr<FlightInfo>> FlightClient::GetFlightInfo(
   return info;
 }
 
-Status FlightClient::GetFlightInfo(const FlightCallOptions& options,
-                                   const FlightDescriptor& descriptor,
-                                   std::unique_ptr<FlightInfo>* info) {
-  return GetFlightInfo(options, descriptor).Value(info);
+void FlightClient::GetFlightInfoAsync(
+    const FlightCallOptions& options, const FlightDescriptor& descriptor,
+    std::shared_ptr<AsyncListener<FlightInfo>> listener) {
+  if (auto status = CheckOpen(); !status.ok()) {
+    listener->OnFinish(std::move(status));
+    return;
+  }
+  transport_->GetFlightInfoAsync(options, descriptor, std::move(listener));
+}
+
+arrow::Future<FlightInfo> FlightClient::GetFlightInfoAsync(
+    const FlightCallOptions& options, const FlightDescriptor& descriptor) {
+  RETURN_NOT_OK(CheckOpen());
+  auto [listener, future] = UnaryUnaryAsyncListener<FlightInfo>::Make();
+  transport_->GetFlightInfoAsync(options, descriptor, std::move(listener));
+  return future;
+}
+
+arrow::Result<std::unique_ptr<PollInfo>> FlightClient::PollFlightInfo(
+    const FlightCallOptions& options, const FlightDescriptor& descriptor) {
+  std::unique_ptr<PollInfo> info;
+  RETURN_NOT_OK(CheckOpen());
+  RETURN_NOT_OK(transport_->PollFlightInfo(options, descriptor, &info));
+  return info;
 }
 
 arrow::Result<std::unique_ptr<SchemaResult>> FlightClient::GetSchema(
@@ -626,20 +654,10 @@ arrow::Result<std::unique_ptr<SchemaResult>> FlightClient::GetSchema(
   return transport_->GetSchema(options, descriptor);
 }
 
-Status FlightClient::GetSchema(const FlightCallOptions& options,
-                               const FlightDescriptor& descriptor,
-                               std::unique_ptr<SchemaResult>* schema_result) {
-  return GetSchema(options, descriptor).Value(schema_result);
-}
-
 arrow::Result<std::unique_ptr<FlightListing>> FlightClient::ListFlights() {
   return ListFlights({}, {});
 }
 
-Status FlightClient::ListFlights(std::unique_ptr<FlightListing>* listing) {
-  return ListFlights({}, {}).Value(listing);
-}
-
 arrow::Result<std::unique_ptr<FlightListing>> FlightClient::ListFlights(
     const FlightCallOptions& options, const Criteria& criteria) {
   std::unique_ptr<FlightListing> listing;
@@ -648,12 +666,6 @@ arrow::Result<std::unique_ptr<FlightListing>> FlightClient::ListFlights(
   return listing;
 }
 
-Status FlightClient::ListFlights(const FlightCallOptions& options,
-                                 const Criteria& criteria,
-                                 std::unique_ptr<FlightListing>* listing) {
-  return ListFlights(options, criteria).Value(listing);
-}
-
 arrow::Result<std::unique_ptr<FlightStreamReader>> FlightClient::DoGet(
     const FlightCallOptions& options, const Ticket& ticket) {
   RETURN_NOT_OK(CheckOpen());
@@ -668,11 +680,6 @@ arrow::Result<std::unique_ptr<FlightStreamReader>> FlightClient::DoGet(
   return stream_reader;
 }
 
-Status FlightClient::DoGet(const FlightCallOptions& options, const Ticket& ticket,
-                           std::unique_ptr<FlightStreamReader>* stream) {
-  return DoGet(options, ticket).Value(stream);
-}
-
 arrow::Result<FlightClient::DoPutResult> FlightClient::DoPut(
     const FlightCallOptions& options, const FlightDescriptor& descriptor,
     const std::shared_ptr<Schema>& schema) {
@@ -689,17 +696,6 @@ arrow::Result<FlightClient::DoPutResult> FlightClient::DoPut(
   return result;
 }
 
-Status FlightClient::DoPut(const FlightCallOptions& options,
-                           const FlightDescriptor& descriptor,
-                           const std::shared_ptr<Schema>& schema,
-                           std::unique_ptr<FlightStreamWriter>* writer,
-                           std::unique_ptr<FlightMetadataReader>* reader) {
-  ARROW_ASSIGN_OR_RAISE(auto result, DoPut(options, descriptor, schema));
-  *writer = std::move(result.writer);
-  *reader = std::move(result.reader);
-  return Status::OK();
-}
-
 arrow::Result<FlightClient::DoExchangeResult> FlightClient::DoExchange(
     const FlightCallOptions& options, const FlightDescriptor& descriptor) {
   RETURN_NOT_OK(CheckOpen());
@@ -717,14 +713,45 @@ arrow::Result<FlightClient::DoExchangeResult> FlightClient::DoExchange(
   return result;
 }
 
-Status FlightClient::DoExchange(const FlightCallOptions& options,
-                                const FlightDescriptor& descriptor,
-                                std::unique_ptr<FlightStreamWriter>* writer,
-                                std::unique_ptr<FlightStreamReader>* reader) {
-  ARROW_ASSIGN_OR_RAISE(auto result, DoExchange(options, descriptor));
-  *writer = std::move(result.writer);
-  *reader = std::move(result.reader);
-  return Status::OK();
+::arrow::Result<SetSessionOptionsResult> FlightClient::SetSessionOptions(
+    const FlightCallOptions& options, const SetSessionOptionsRequest& request) {
+  RETURN_NOT_OK(CheckOpen());
+  ARROW_ASSIGN_OR_RAISE(auto body, request.SerializeToString());
+  Action action{ActionType::kSetSessionOptions.type, Buffer::FromString(body)};
+  ARROW_ASSIGN_OR_RAISE(auto stream, DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(auto result, stream->Next());
+  ARROW_ASSIGN_OR_RAISE(
+      auto set_session_options_result,
+      SetSessionOptionsResult::Deserialize(std::string_view(*result->body)));
+  ARROW_RETURN_NOT_OK(stream->Drain());
+  return set_session_options_result;
+}
+
+::arrow::Result<GetSessionOptionsResult> FlightClient::GetSessionOptions(
+    const FlightCallOptions& options, const GetSessionOptionsRequest& request) {
+  RETURN_NOT_OK(CheckOpen());
+  ARROW_ASSIGN_OR_RAISE(auto body, request.SerializeToString());
+  Action action{ActionType::kGetSessionOptions.type, Buffer::FromString(body)};
+  ARROW_ASSIGN_OR_RAISE(auto stream, DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(auto result, stream->Next());
+  ARROW_ASSIGN_OR_RAISE(
+      auto get_session_options_result,
+      GetSessionOptionsResult::Deserialize(std::string_view(*result->body)));
+  ARROW_RETURN_NOT_OK(stream->Drain());
+  return get_session_options_result;
+}
+
+::arrow::Result<CloseSessionResult> FlightClient::CloseSession(
+    const FlightCallOptions& options, const CloseSessionRequest& request) {
+  RETURN_NOT_OK(CheckOpen());
+  ARROW_ASSIGN_OR_RAISE(auto body, request.SerializeToString());
+  Action action{ActionType::kCloseSession.type, Buffer::FromString(body)};
+  ARROW_ASSIGN_OR_RAISE(auto stream, DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(auto result, stream->Next());
+  ARROW_ASSIGN_OR_RAISE(auto close_session_result,
+                        CloseSessionResult::Deserialize(std::string_view(*result->body)));
+  ARROW_RETURN_NOT_OK(stream->Drain());
+  return close_session_result;
 }
 
 Status FlightClient::Close() {
@@ -736,6 +763,10 @@ Status FlightClient::Close() {
   return Status::OK();
 }
 
+bool FlightClient::supports_async() const { return transport_->CheckAsyncSupport().ok(); }
+
+Status FlightClient::CheckAsyncSupport() const { return transport_->CheckAsyncSupport(); }
+
 Status FlightClient::CheckOpen() const {
   if (closed_) {
     return Status::Invalid("FlightClient is closed");
