@@ -20,6 +20,7 @@
 
 #pragma once
 
+#include <map>
 #include <memory>
 #include <optional>
 #include <string>
@@ -590,7 +591,7 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
 
   /// \brief Commit/rollback a transaction.
   /// \param[in] context  The call context.
-  /// \param[in] request  The tranaction.
+  /// \param[in] request  The transaction.
   virtual Status EndTransaction(const ServerCallContext& context,
                                 const ActionEndTransactionRequest& request);
 
@@ -601,6 +602,24 @@ class ARROW_FLIGHT_SQL_EXPORT FlightSqlServerBase : public FlightServerBase {
   virtual arrow::Result<CancelFlightInfoResult> CancelFlightInfo(
       const ServerCallContext& context, const CancelFlightInfoRequest& request);
 
+  /// \brief Set server session option(s).
+  /// \param[in] context  The call context.
+  /// \param[in] request  The session options to set.
+  virtual arrow::Result<SetSessionOptionsResult> SetSessionOptions(
+      const ServerCallContext& context, const SetSessionOptionsRequest& request);
+
+  /// \brief Get server session option(s).
+  /// \param[in] context  The call context.
+  /// \param[in] request  Request object.
+  virtual arrow::Result<GetSessionOptionsResult> GetSessionOptions(
+      const ServerCallContext& context, const GetSessionOptionsRequest& request);
+
+  /// \brief Close/invalidate the session.
+  /// \param[in] context  The call context.
+  /// \param[in] request  Request object.
+  virtual arrow::Result<CloseSessionResult> CloseSession(
+      const ServerCallContext& context, const CloseSessionRequest& request);
+
   /// \brief Attempt to explicitly cancel a query.
   ///
   /// \param[in] context  The call context.
